package info.bukova.isspst.services.requirement;

import info.bukova.isspst.Constants;
import info.bukova.isspst.data.JoinedItem;
import info.bukova.isspst.data.Requirement;
import info.bukova.isspst.data.RequirementItem;
import info.bukova.isspst.data.RequirementState;
import info.bukova.isspst.data.User;
import info.bukova.isspst.data.Workflow;
import info.bukova.isspst.data.Workgroup;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.hibernate.Query;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PostFilter;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;

public class RequirementServiceImpl extends
		RequirementBaseServiceImpl<Requirement> implements RequirementService,
		RequirementBaseService<Requirement> {

	@Autowired
	private RequirementTypeService reqTypeService;

	@Override
	protected Requirement createEntity() {
		Requirement entity = new Requirement();

		entity.setReqDate(new Date());
		entity.setType(reqTypeService.getTypeById(Constants.REQTYPE_ORDER));
		entity.setState(RequirementState.NEW);

		return entity;
	}

	@Override
	protected boolean autoApprove(Requirement entity) {
		List<User> approvers = this.getNextApprover(entity);
		Workflow nextWf = this.getNextWorkflow(entity);
		
		if (approvers == null || approvers.isEmpty() || nextWf == null || nextWf.getLimit() == null) {
			return false;
		}
		
		if (entity.getSumTotal().compareTo(nextWf.getLimit()) == -1) {
			approve(entity, approvers.get(0));
			return true;
		}
		
		return false;
	}

	@SuppressWarnings("unchecked")
	@Override
	@Transactional
	@PreAuthorize("hasPermission(this, 'PERM_SHOW_CENTRE_REQ')")
	@PostFilter("hasPermission(filterObject, 'PERM_SHOW_CENTRE_REQ')")
	public List<JoinedItem> getItemsForOrder() {
		List<Workgroup> wgList = workgroupService.getUserCentres(getLoggedInUser());
		Query q = dao.getQuery("select item from RequirementItem item left join item.requirement rq join rq.centre c where rq.kind is not null and rq.state = :state and c in (:wgList)");
		q.setParameterList("wgList", wgList);		
		q.setParameter("state", RequirementState.APPROVED);
		List<JoinedItem> items = new ArrayList<JoinedItem>();

		for (RequirementItem it : (List<RequirementItem>)q.list()) {
			items.add(new JoinedItem(it, it.getRequirement().getWorkgroup(), it.getRequirement().getCentre(), it.getRequirement().getOwnedBy()));
		}
		
		return items;
	}

}
